//
//  SearchResultsHTMLController.m
//  HelTweetica
//
//  Created by Lucius Kwok on 5/23/10.

/*
 Copyright (c) 2010, Felt Tip Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:  
 1.  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 2.  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 3.  Neither the name of the copyright holder(s) nor the names of any contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "SearchResultsHTMLController.h"
#import "TwitterSearchAction.h"
#import "NSString+HTMLFormatted.h"



@implementation SearchResultsHTMLController


// Designated initializer.
- (id)initWithQuery:(NSString*)aQuery twitter:(Twitter *)twttr {
	self = [super init];
	if (self) {
		self.customPageTitle = [NSString stringWithFormat: @"Search for &ldquo;<b>%@</b>&rdquo;", [aQuery HTMLFormatted]];
		self.customTabName = NSLocalizedString (@"Results", @"tab");
		maxTweetsShown = 1000; // Allow for a larger limit for searches.
		
		// Timeline
		self.timeline = [[[TwitterTimeline alloc] init] autorelease]; // Always start with an empty array of messages for Search.
		timeline.loadAction = [[[TwitterSearchAction alloc] initWithQuery:aQuery] autorelease];
		timeline.loadAction.countKey = @"rpp";
		timeline.replaceExistingStatusUpdates = NO;

		// Database
		[timeline setTwitter:twttr tableName:[NSString stringWithFormat:@"SearchResults_%u", [aQuery hash]] temp:YES];

		// Notifications
		NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(timelineDidFinishLoading:) name:TwitterTimelineDidFinishLoadingNotification object:nil];
		
		[self loadTimeline:timeline];
		
	}
	return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}	


#pragma mark HTML formatting

- (NSString*) webPageTemplate {
	return [self loadHTMLTemplate:@"basic-template"];
}

#pragma mark Notifications

- (void)timelineDidFinishLoading:(NSNotification *)notification {
	TwitterTimeline *aTimeline = [notification object];
	if (aTimeline == self.timeline) {
		isLoading = NO;
		self.messages = [timeline messagesWithLimit: maxTweetsShown];
		[self rewriteTweetArea];
	}
}

@end
